﻿using System;
using System.Collections.Generic;
using System.Web.Mvc;
using Nop.Core;
using Nop.Core.Domain.Payments;
using Nop.Plugin.Payments.AggrepayPayments.Models;
using Nop.Services.Configuration;
using Nop.Services.Orders;
using Nop.Services.Payments;
using Nop.Services.Localization;

using Nop.Web.Framework.Controllers;
using Nop.Services.Stores;
using Nop.Web.Framework;
using Nop.Services.Logging;

namespace Nop.Plugin.Payments.AggrepayPayments.Controllers
{
    public class PaymentAggrepayPaymentsController : BasePaymentController
    {
        private readonly ISettingService _settingService;
        private readonly IPaymentService _paymentService;
        private readonly IOrderService _orderService;
        private readonly IOrderProcessingService _orderProcessingService;
        private readonly AggrepayPaymentsPaymentSettings _AggrepayPaymentsPaymentSettings;
        private readonly PaymentSettings _paymentSettings;
        private readonly ILogger _logger;
        private readonly ILocalizationService _localizationService;
        public PaymentAggrepayPaymentsController(ISettingService settingService,
        IPaymentService paymentService, IOrderService orderService,
             IOrderProcessingService orderProcessingService,
             ILocalizationService localizationService,
            AggrepayPaymentsPaymentSettings AggrepayPaymentsPaymentSettings,
            PaymentSettings paymentSettings, ILogger logger)
        {
            this._settingService = settingService;
            this._paymentService = paymentService;
            this._orderService = orderService;
            this._orderProcessingService = orderProcessingService;
            this._AggrepayPaymentsPaymentSettings = AggrepayPaymentsPaymentSettings;
            this._localizationService = localizationService;
            this._paymentSettings = paymentSettings;
            this._logger = logger;

        }


        [AdminAuthorize]
        [ChildActionOnly]
        public ActionResult Configure()
        {
            var model = new ConfigurationModel();
            model.Api_Key = _AggrepayPaymentsPaymentSettings.Api_Key;
            model.Salt = _AggrepayPaymentsPaymentSettings.Salt;
            model.TransactModeId = Convert.ToInt32(_AggrepayPaymentsPaymentSettings.TransactMode);
            model.TransactModeValues = _AggrepayPaymentsPaymentSettings.TransactMode.ToSelectList();

            return View("~/Plugins/Payments.AggrepayPayments/Views/PaymentAggrepayPayments/Configure.cshtml", model);
        }

        [HttpPost]
        [AdminAuthorize]
        [ChildActionOnly]
        public ActionResult Configure(ConfigurationModel model)
        {
            if (!ModelState.IsValid)
                return Configure();

            //save settings
            _AggrepayPaymentsPaymentSettings.Api_Key = model.Api_Key;
            _AggrepayPaymentsPaymentSettings.Salt = model.Salt;
            _AggrepayPaymentsPaymentSettings.TransactMode = (TransactMode)model.TransactModeId;

            _settingService.SaveSetting(_AggrepayPaymentsPaymentSettings);

            SuccessNotification(_localizationService.GetResource("Admin.Plugins.Saved"));

            return Configure();
        }

        [ChildActionOnly]
        public ActionResult PaymentInfo()
        {
            var model = new PaymentInfoModel();
            return View("~/Plugins/Payments.AggrepayPayments/Views/PaymentAggrepayPayments/PaymentInfo.cshtml", model);

        }

        [NonAction]
        public override IList<string> ValidatePaymentForm(FormCollection form)
        {
            var warnings = new List<string>();
            return warnings;
        }

        [NonAction]
        public override ProcessPaymentRequest GetPaymentInfo(FormCollection form)
        {
            var paymentInfo = new ProcessPaymentRequest();
            return paymentInfo;
        }

        [ValidateInput(false)]
        public ActionResult Return(FormCollection form)
        {
            var processor = _paymentService.LoadPaymentMethodBySystemName("Payments.AggrepayPayments") as AggrepayPaymentsPaymentProcessor;
            if (processor == null ||
                !processor.IsPaymentMethodActive(_paymentSettings) || !processor.PluginDescriptor.Installed)
                throw new NopException("AggrepayPayments module cannot be loaded");


            var myUtility = new AggrepayPaymentsHelper();

            string[] hash_columns = Request.Form.AllKeys;
            Array.Sort(hash_columns);
            IDictionary<string, string> data = new Dictionary<string, string>();

            foreach (string column in hash_columns)
            {
                data.Add(column, form[column]);    
            }
            data["hash"] = "";
            string response_code, server_hash, hash,hash_data, response_message;    

            //Assign following values to send it to verifychecksum function.
            if (String.IsNullOrWhiteSpace(_AggrepayPaymentsPaymentSettings.Salt))
                throw new NopException("AggrepayPayments Salt is not set");

            data.Add("salt", _AggrepayPaymentsPaymentSettings.Salt.ToString());
            server_hash = form["hash"];
            response_code = form["response_code"];
            response_message = form["response_message"];

            string[] hash_values = myUtility.gethash(hash_columns, data);
            hash_values[0] = hash_values[0].Remove(0, 40);
            hash_values[0] = hash_values[0].Insert(0, "****************SALT********************");

            hash_data = hash_values[0];
            hash = hash_values[1];

            if (hash == server_hash)
            {

			 if(response_code == "0")
			 {
                var order = _orderService.GetOrderById(Convert.ToInt32(data["order_id"]));
                if (_orderProcessingService.CanMarkOrderAsPaid(order))
                {
                    _orderProcessingService.MarkOrderAsPaid(order);
                }
                    _logger.Information("Payment ₹" + data["amount"] + " successful for Order Id " + data["order_id"]+ " and Transaction Id "+form["transaction_id"]);
                    //Thank you for shopping with us. Your credit card has been charged and your transaction is successful
                    return RedirectToRoute("CheckoutCompleted", new { orderId = order.Id});
			  }
             else if (response_message != "0")
                {
                    _logger.Error("Payment unsuccessful for Order Id " + data["order_id"]);
                    return View("~/Plugins/Payments.AggrepayPayments/Views/PaymentAggrepayPayments/Failure.cshtml");
                }
			  
			  else
			  {
                    var errorModel = new ErrorModel();
                    errorModel.Error = response_message;
                    return View("~/Plugins/Payments.AggrepayPayments/Views/PaymentAggrepayPayments/Error.cshtml",errorModel);

                }

            }
            
            
            else
            {
                _logger.Error("Hash mis matched !,Hash Calculated with data sent by server is : "+ hash_data);
                return View("~/Plugins/Payments.AggrepayPayments/Views/PaymentAggrepayPayments/Hash_Failure.cshtml");
            }
        }
    }
}